/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.urlhandlers;

import org.wso2.registry.Artifact;
import org.wso2.registry.Comment;
import org.wso2.registry.RegistryException;
import org.wso2.registry.jdbc.dao.CommentsDAO;
import org.wso2.registry.jdbc.ConnectionFactory;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.sql.Connection;
import java.sql.SQLException;

public class CommentURLHandler extends URLHandler {

    private static final Log log = LogFactory.getLog(CommentURLHandler.class);

    private CommentsDAO commentsDAO = new CommentsDAO();

    public CommentURLHandler(ConnectionFactory connectionFactory) {
        super(connectionFactory);
    }

    /**
     * Accepts URL in the form <path>?comment<comment-id>
     *
     * @param url
     * @param artifact
     * @return
     */
    public boolean handleURL(String url, Artifact artifact) throws RegistryException {

        String[] parts = url.split("\\?");

        if (parts.length != 2) {
            return false;
        }

        if (parts[0].length() == 0) {
            return false;
        }

        if (!parts[1].startsWith("comment")) {
            return false;
        }

        String commentID = parts[1].substring("comment".length());
        if (!(commentID.length() > 0)) {
            return false;
        }

        long cID = 0;
        try {
            cID = Long.parseLong(commentID);

        } catch (NumberFormatException e) {
            // note that this might not be an exceptional scenario. there could be a different URL
            // form, which contains strings after "comment".
            // it is just that it is not the URL we expect here
            return false;
        }

        Connection conn = connectionFactory.getConnection();

        Comment comment = null;
        try {
            comment = commentsDAO.getComment(cID, conn);

        } catch (SQLException e) {
            String msg = "Could not get the comment with ID: " + cID +
                    ". Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);

        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException ignore) {}
            }
        }

        if (comment == null) {
            String msg = "Requested comment with ID: " + cID + " is not available.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        artifact.setPath(url);
        artifact.setContent(comment.getCommentText().getBytes());
        artifact.setAuthorUserName(comment.getCommentedUser());
        artifact.setCreatedTime(comment.getCommentedTime());

        return true;
    }
}
